# -*- coding: utf-8 -*-

from __future__ import (absolute_import, division, print_function)

import array
import math
import itertools
import pytest

from symcxx import NameSpace

# Tests I wrote for Lambdify in symengine reference 'se'
se = NameSpace()

s0, s1, s2 = map(se.Symbol, 's0,s1,s2'.split(','))


def test_single_arg():
    f = se.lambdify(s0, 2*s0)
    assert f(1) == 2


def test_list_args():
    f = se.lambdify([s0, s1], s0 + s1)
    assert f(1, 2) == 3


def test_sin():
    f = se.lambdify(s0, se.sin(s0))
    assert f(0) == 0.


def test_exponentiation():
    f = se.lambdify(s0, s0**2)
    assert f(-1) == 1
    assert f(0) == 0
    assert f(1) == 1
    assert f(-2) == 4
    assert f(2) == 4
    assert f(2.5) == 6.25


def test_sqrt():
    f = se.lambdify(s0, se.sqrt(s0))
    assert f(0) == 0.
    assert f(1) == 1.
    assert f(4) == 2.
    assert abs(f(2) - 1.414) < 0.001
    assert f(6.25) == 2.5


def test_trig():
    f = se.lambdify([s0], [se.cos(s0), se.sin(s0)])
    d = f(math.pi)
    prec = 1e-11
    assert -prec < d[0] + 1
    assert d[0] + 1 < prec
    assert -prec < d[1]
    assert d[1] < prec
    d = f(3.14159)
    prec = 1e-5
    assert -prec < d[0] + 1
    assert d[0] < prec
    assert -prec < d[1]
    assert d[1] < prec


def allclose(iter_a, iter_b, rtol=1e-10, atol=1e-10):
    for a, b in zip(iter_a, iter_b):
        if not (abs(a-b) < abs(a)*rtol + atol):
            return False
    return True


def test_vector_simple():
    f = se.lambdify((s0, s1, s2), (s2, s1, s0))
    assert allclose(f(3, 2, 1), (1, 2, 3))
    assert allclose(f(1., 2., 3.), (3., 2., 1.))
    with pytest.raises(TypeError):
        f(0)


def test_trig_symbolic():
    f = se.lambdify([s0], [se.cos(s0), se.sin(s0)])
    d = f(math.pi)
    assert abs(d[0] + 1) < 0.0001
    assert abs(d[1] - 0) < 0.0001


def test_trig_float():
    f = se.lambdify([s0], [se.cos(s0), se.sin(s0)])
    d = f(3.14159)
    assert abs(d[0] + 1) < 0.0001
    assert abs(d[1] - 0) < 0.0001


def test_Lambdift_simple():
    h = se.lambdify([s2], [s2])
    assert h(8) == 8
    x = se.Symbol('x')
    i = se.lambdify([x], [x])
    assert i(16) == 16


def test_Lambdify_twice():
    n = 7

    args1 = x, y = se.symbols('x y')

    exprs1 = [x+y, x*y]

    l1 = se.Lambdify(args1, exprs1)

    v1 = l1(range(n, n+len(args1)))
    ref1 = [2*n+1, n*(n+1)]
    assert allclose(v1, ref1)

    args2 = u, v = se.symbols('u v')

    exprs2 = [u - v, u/v]

    mat = se.Matrix(len(exprs2), 1, exprs2)

    mat2 = se.rebuild_debug(args2, mat)
    assert mat2.ns.n_symbs == 2

    l2 = se.Lambdify(args2, exprs2)
    v2 = l2(range(n, n+len(args2)))
    ref2 = [-1, n/(n+1)]
    assert allclose(v2, ref2)


def test_Lambdify():
    n = 7
    args = x, y, z = se.symbols('x y z')
    l = se.Lambdify(args, [x+y+z, x**2, (x-y)/z, x*y*z])
    assert allclose(l(range(n, n+len(args))),
                    [3*n+3, n**2, -1/(n+2), n*(n+1)*(n+2)])


def test_Lambdify_out():
    import numpy as np
    n = 7
    args = x, y, z = se.symbols('x y z')
    exprs = [x+y+z, x**2, (x-y)/z, x*y*z]
    lmb = se.Lambdify(args, exprs)
    inp = np.arange(n, n+6.)
    out = np.empty(len(exprs))
    lmb(inp, out)
    assert allclose(out,
                    [3*n+3, n**2, -1/(n+2), n*(n+1)*(n+2)])


def _get_2_to_2by2_numpy():
    import numpy as np
    args = x, y = se.symbols('x y')
    exprs = np.array([[x+y+1.0, x*y],
                      [x/y, x**y]])
    l = se.Lambdify(args, exprs)

    def check(A, inp):
        X, Y = inp
        assert abs(A[0, 0] - (X+Y+1.0)) < 1e-15
        assert abs(A[0, 1] - (X*Y)) < 1e-15
        assert abs(A[1, 0] - (X/Y)) < 1e-15
        assert abs(A[1, 1] - (X**Y)) < 1e-13
    return l, check


def test_Lambdify_2dim_numpy():
    import numpy as np
    lmb, check = _get_2_to_2by2_numpy()
    for inp in [(5, 7), np.array([5, 7]), [5.0, 7.0]]:
        A = lmb(inp)
        assert A.shape == (2, 2)
        check(A, inp)


def _get_array():
    X, Y, Z = inp = array.array('d', [1, 2, 3])
    args = x, y, z = se.symbols('x y z')
    exprs = [x+y+z, se.sin(x)*se.log(y)*se.exp(z)]
    ref = [X+Y+Z, math.sin(X)*math.log(Y)*math.exp(Z)]

    def check(arr):
        assert all([abs(x1-x2) < 1e-13 for x1, x2 in zip(ref, arr)])
    return args, exprs, inp, check


def test_array():
    args, exprs, inp, check = _get_array()
    lmb = se.Lambdify(args, exprs)
    out = lmb(inp)
    check(out)


def _get_1_to_2by3_matrix():
    x = se.symbols('x')
    args = x,
    exprs = se.Matrix(2, 3, [x+1, x+2, x+3,
                             1/x, 1/(x*x), 1/(x**3.0)])
    l = se.Lambdify(args, exprs)

    def check(A, inp):
        X, = inp
        assert abs(A[0, 0] - (X+1)) < 1e-15
        assert abs(A[0, 1] - (X+2)) < 1e-15
        assert abs(A[0, 2] - (X+3)) < 1e-15
        assert abs(A[1, 0] - (1/X)) < 1e-15
        assert abs(A[1, 1] - (1/(X*X))) < 1e-15
        assert abs(A[1, 2] - (1/(X**3.0))) < 1e-15
    return l, check


def test_x_times_x():
    l = se.Lambdify([s0], [s0*s0])
    res = l([7.0])
    assert abs(res - 49.) < 1e-15


def test_2dim_Matrix():
    l, check = _get_1_to_2by3_matrix()
    inp = [7]
    check(l(inp), inp)


def test_lambdify_jacobian_simple():
    import numpy as np
    args = se.Matrix(1, 1, [s0])
    v = se.Matrix(1, 1, [s0*s0])
    jac = v.jacobian(args)
    return
    lmb = se.Lambdify(args, jac)
    out = np.empty((1, 1))
    inp = X, = 7,
    lmb(inp, out)
    assert np.atleast_1d(out).shape == (1, 1)
    assert np.allclose(out, [[14.0]])


def test_lambdify_jacobian0():
    import numpy as np
    args = se.Matrix(2, 1, [s0, s1])
    v = se.Matrix(2, 1, [s0**3 * s1, (s0+1)*(s1+1)])
    jac = v.jacobian(args)
    lmb = se.Lambdify([s0, s1], jac)
    out = np.empty((2, 2))
    inp = X, Y = 7, 11
    lmb(inp, out)
    assert out.shape == (2, 2)
    assert np.allclose(out, [[3 * X**2 * Y, X**3],
                             [Y + 1, X + 1]])


def test_lambdify_jacobian1():
    import numpy as np
    x, y = se.symbols('x, y')
    args = se.Matrix(2, 1, [x, y])
    v = se.Matrix(2, 1, [x**3 * y, (x+1)*(y+1)])
    jac = v.jacobian(args)
    lmb = se.Lambdify(args, jac)
    out = np.empty((2, 2))
    inp = X, Y = 7, 11
    lmb(inp, out)
    assert out.shape == (2, 2)
    assert np.allclose(out, [[3 * X**2 * Y, X**3],
                             [Y + 1, X + 1]])


def test_lambdify_jacobian2():
    import numpy as np
    x, y = se.symbols('x, y')
    args = se.Matrix(2, 1, [x, y])
    v = se.Matrix(2, 1, [x**3 * y, (x+1)*(y+1)])
    jac = v.jacobian(args)
    lmb = se.Lambdify(args, jac)
    inp = X, Y = 7, 11
    out = lmb(inp)
    assert np.atleast_1d(out).shape == (2, 2)
    assert np.allclose(out, [[3 * X**2 * Y, X**3],
                             [Y + 1, X + 1]])


def test_itertools_chain():
    args, exprs, inp, check = _get_array()
    l = se.Lambdify(args, exprs)
    inp = itertools.chain([inp[0]], (inp[1],), [inp[2]])
    A = l(tuple(inp))  # numpy 1.10 array failing for itertools chain
    check(A)



# -*- coding: utf-8 -*-
from __future__ import absolute_import, division, print_function

import math
import numpy as np
from symcxx import NameSpace


def test_NameSpace_Symbol():
    ns = NameSpace(2)
    x = ns.Symbol('x')
    y = ns.Symbol('y')
    add = x + y
    result = add.evalf(np.array([3., 5.]))
    assert abs(result - 8) < 1e-15

    sub = x - y
    result = sub.evalf(np.array([3., 5.]))
    assert abs(result + 2) < 1e-15


def test_NameSpace_relational():
    ns = NameSpace(2)
    x = ns.Symbol('x')
    y = ns.Symbol('y')
    lt = ns.lt(x, y)
    le = ns.le(x, y)
    eq = ns.eq(x, y)
    ne = ns.ne(x, y)
    ge = ns.ge(x, y)
    gt = ns.gt(x, y)
    assert lt.evalb(np.array([3., 5.])) is True
    assert lt.evalb(np.array([5., 5.])) is False
    assert le.evalb(np.array([3., 5.])) is True
    assert le.evalb(np.array([5., 5.])) is True
    assert eq.evalb(np.array([3., 5.])) is False
    assert eq.evalb(np.array([5., 5.])) is True
    assert ne.evalb(np.array([3., 5.])) is True
    assert ne.evalb(np.array([5., 5.])) is False
    assert ge.evalb(np.array([3., 5.])) is False
    assert ge.evalb(np.array([5., 5.])) is True
    assert gt.evalb(np.array([3., 5.])) is False
    assert gt.evalb(np.array([5., 5.])) is False


def test_division():
    ns = NameSpace(2)
    assert ns.symbol_names == []

    x, y = map(ns.Symbol, 'x y'.split())
    zero_over_x_1 = 0/x
    zero_over_x_2 = 0/x
    assert zero_over_x_1 == zero_over_x_2

    expr = x/y
    assert abs(expr.evalf(np.array([3., 7.])) - 3/7) < 1e-15

    assert x/3 == x/3
    assert 3/y == 3/y
    assert 3/x != 3/y


def test_diff0():
    ns = NameSpace(1)
    x = ns.Symbol('x')
    expr = (3*x)
    deriv = expr.diff(x)
    ref = ns.Number(3)
    assert (deriv == ref) is True


def test_str():
    ns = NameSpace(0)
    x = ns.Symbol('x')
    assert str(3*x) in ('Mul2(Integer(3), Symbol(x))',
                        'Mul2(Symbol(x), Integer(3))')


def test_autosimplification():
    ns = NameSpace(0)
    x = ns.Symbol('x')
    assert x - x == 0
    assert x**2 - x*x == 0
    # assert x**2*x**3 - x*x**4 == 0
    # assert x*x*x*x - x**4 == 0
    # assert x*4 - 2*(x*2) == 0
    # assert x*4 - x - x - x - x == 0


def test_trigfuncs_evalf():
    ns = NameSpace()
    cos_pi = ns.cos(ns.pi)
    sin_pi = ns.sin(ns.pi)
    tan_pi = ns.tan(ns.pi)
    assert abs(cos_pi.evalf(np.array([])) + 1) < 1e-15
    assert abs(sin_pi.evalf(np.array([]))) < 1e-15
    assert abs(tan_pi.evalf(np.array([]))) < 1e-15


def test_Div_diff0():
    ns = NameSpace(1)
    x = ns.Symbol('x')
    expr = (3*x + 1) / (x - 2)
    deriv = expr.diff(x)
    ref = (3*(3.14-2) - (3*3.14 + 1)) / ((3.14 - 2)**2)
    assert abs(deriv.evalf(np.array([3.14])) - ref) < 1e-15


def test_Pow_diff0():
    ns = NameSpace(7)
    x = ns.Symbol('x')
    expr = (3*x + 1) ** (x - 2)
    deriv = expr.diff(x)
    assert abs(deriv.evalf(np.array([3.14])) - 38.6541588883393) < 1e-13


def test_sin_diff0():
    ns = NameSpace(1)
    x = ns.Symbol('x')
    expr = ns.sin(3*x + 1)
    ref = 3*math.cos(3*3.14 + 1)
    assert abs(expr.diff(x).evalf(np.array([3.14])) - ref) < 1e-15


def test_cos_diff0():
    ns = NameSpace(1)
    x = ns.Symbol('x')
    expr = ns.cos(3*x + 1)
    ref = -3*math.sin(3*3.14 + 1)
    assert abs(expr.diff(x).evalf(np.array([3.14])) - ref) < 1e-15


def test_exp_diff0():
    ns = NameSpace(1)
    x = ns.Symbol('x')
    expr = ns.exp(3*x + 1)
    ref = 3*math.exp(3*3.14 + 1)
    assert abs(expr.diff(x).evalf(np.array([3.14])) - ref) < 1e-15


def test_tan_diff0():
    ns = NameSpace(1)
    x = ns.Symbol('x')
    expr = ns.tan(3*x + 1)
    ref = 3*(1 + math.tan(3*3.14 + 1)**2)
    assert abs(expr.diff(x).evalf(np.array([3.14])) - ref) < 1e-15


def test_acos_diff0():
    ns = NameSpace(1)
    x = ns.Symbol('x')
    expr = ns.acos(.1*x + .2)
    ref = -0.11657862476093600653
    assert abs(expr.diff(x).evalf(np.array([3.14])) - ref) < 1e-15


def test_asin_diff0():
    ns = NameSpace(1)
    x = ns.Symbol('x')
    expr = ns.asin(.1*x + .2)
    ref = 0.11657862476093600653
    assert abs(expr.diff(x).evalf(np.array([3.14])) - ref) < 1e-15


def test_atan_diff0():
    ns = NameSpace(1)
    x = ns.Symbol('x')
    expr = ns.atan(.1*x + .2)
    ref = .1/((.1*3.14+.2)**2 + 1)
    assert abs(expr.diff(x).evalf(np.array([3.14])) - ref) < 1e-15


def test_cosh_diff0():
    ns = NameSpace(1)
    x = ns.Symbol('x')
    expr = ns.cosh(.1*x + .2)
    ref = .1*math.sinh(.1*3.14 + .2)
    assert abs(expr.diff(x).evalf(np.array([3.14])) - ref) < 1e-15


def test_sinh_diff0():
    ns = NameSpace(1)
    x = ns.Symbol('x')
    expr = ns.sinh(.1*x + .2)
    ref = .1*math.cosh(.1*3.14 + .2)
    assert abs(expr.diff(x).evalf(np.array([3.14])) - ref) < 1e-15


def test_tanh_diff0():
    ns = NameSpace(1)
    x = ns.Symbol('x')
    expr = ns.tanh(.1*x + .2)
    ref = .1*(1 - math.tanh(.1*3.14 + .2)**2)
    assert abs(expr.diff(x).evalf(np.array([3.14])) - ref) < 1e-15


def test_acosh_diff0():
    ns = NameSpace(1)
    x = ns.Symbol('x')
    expr = ns.acosh(3*x + 4)
    ref = 0.224170172808993
    assert abs(expr.diff(x).evalf(np.array([3.14])) - ref) < 1e-15


def test_asinh_diff0():
    ns = NameSpace(1)
    x = ns.Symbol('x')
    expr = ns.asinh(3*x + 4)
    ref = 0.222928886183630
    assert abs(expr.diff(x).evalf(np.array([3.14])) - ref) < 1e-15


def test_atanh_diff0():
    ns = NameSpace(1)
    x = ns.Symbol('x')
    expr = ns.atanh(3*x + 4)
    ref = -0.0167507554590712
    assert abs(expr.diff(x).evalf(np.array([3.14])) - ref) < 1e-15


def test_log_diff0():
    ns = NameSpace(1)
    x = ns.Symbol('x')
    expr = ns.log(3*x + 1)
    ref = 3/(3*3.14 + 1)
    assert abs(expr.diff(x).evalf(np.array([3.14])) - ref) < 1e-15


def test_log10_diff0():
    ns = NameSpace(1)
    x = ns.Symbol('x')
    expr = ns.log10(3*x + 1)
    ref = 3/(3*3.14 + 1)/math.log(10)
    assert abs(expr.diff(x).evalf(np.array([3.14])) - ref) < 1e-15


def test_exp2_diff0():
    ns = NameSpace(1)
    x = ns.Symbol('x')
    expr = ns.exp2(3*x + 1)
    ref = 3*2**(3*3.14 + 1)*math.log(2)
    assert abs(expr.diff(x).evalf(np.array([3.14])) - ref) < 1e-12


def test_expm1():
    ns = NameSpace(1)
    x = ns.Symbol('x')
    expr = ns.expm1(3*x + 1e-16)
    assert abs(expr.evalf(np.array([3.14e-17])) - 1.942e-16) < 1e-18


def test_expm1_diff0():
    ns = NameSpace(1)
    x = ns.Symbol('x')
    expr = ns.expm1(3*x + 1e-16)
    ref = 3 + 2.86e-14
    assert abs(expr.diff(x).evalf(np.array([3.14e-15])) - ref) < 1e-15


def test_sqrt_diff0():
    ns = NameSpace(1)
    x = ns.Symbol('x')
    expr = ns.sqrt(3*x + 1)
    ref = .5*3/(3*3.14 + 1)**.5
    assert abs(expr.diff(x).evalf(np.array([3.14])) - ref) < 1e-15


def test_cbrt_diff0():
    ns = NameSpace(1)
    x = ns.Symbol('x')
    expr = ns.cbrt(3*x + 1)
    ref = 1./(3*3.14 + 1)**(2/3.)
    assert abs(expr.diff(x).evalf(np.array([3.14])) - ref) < 1e-15


def test_erf_diff0():
    ns = NameSpace(1)
    x = ns.Symbol('x')
    expr = ns.erf(.1*x - 1)
    ref = .1*2*math.exp(-(.1*3.14-1)**2)/math.pi**.5
    assert abs(expr.diff(x).evalf(np.array([3.14])) - ref) < 1e-15


def test_erfc_diff0():
    ns = NameSpace(1)
    x = ns.Symbol('x')
    expr = ns.erfc(.1*x - 1)
    ref = -.1*2*math.exp(-(.1*3.14-1)**2)/math.pi**.5
    assert abs(expr.diff(x).evalf(np.array([3.14])) - ref) < 1e-15


def test_Matrix():
    ns = NameSpace(2)
    x = ns.Symbol('x')
    y = ns.Symbol('y')
    matrix = ns.Matrix(2, 2, [[x+1, y-2],
                              [x-y, x*y]])
    out = matrix.eval_float(np.array([2., 3.]))
    assert np.allclose(out, [[3, 1],
                             [-1, 6]])


def test_jacobian():
    ns = NameSpace(2)
    x = ns.Symbol('x')
    y = ns.Symbol('y')
    my = ns.Matrix(2, 1, [x+y, x*y])
    mx = ns.Matrix(1, 2, [x, y])
    assert my.shape == (2, 1)
    assert mx.shape == (1, 2)
    J = my.jacobian(mx)
    out = J.eval_float(np.array([2., 3.]))
    assert np.allclose(out, [[1, 1],
                             [3, 2]])


def test_Lambdify():
    ns = NameSpace(2)
    x = ns.Symbol('x')
    y = ns.Symbol('y')
    z = ns.Symbol('z')
    u = ns.Symbol('u')
    v = ns.Symbol('v')
    w = ns.Symbol('w')

    lmb0 = ns.Lambdify((x, y, z, u, v, w), [x, y, z, u, v, w])
    out0 = lmb0(np.array([2., 3, 4, 5, 6, 7]))
    ref0 = [2., 3, 4, 5, 6, 7]
    assert np.allclose(out0.flatten(), ref0)

    lmb3 = ns.Lambdify((z, y, x), [x, y, z])
    out3 = lmb3(np.array([2., 3, 4]))
    ref3 = [4, 3, 2]
    assert np.allclose(out3.flatten(), ref3)

    lmb1 = ns.Lambdify((x, y), [x+y, x-y, x*y, 13*x + 2*y])
    out1 = lmb1(np.array([2.0, 3.0]))
    ref1 = [5, -1, 6, 13*2 + 2*3]
    assert np.allclose(out1.flatten(), ref1)

    lmb2 = ns.Lambdify((y, x), [x+y, x-y, x*y])
    out2 = lmb2(np.array([2.0, 3.0]))
    ref2 = [5, 1, 6]
    assert np.allclose(out2.flatten(), ref2)


def test_equality():
    ns = NameSpace()
    p0 = ns.Symbol('p')
    p1 = ns.Symbol('p')
    assert p0 == p1
    assert p1 == p1


def test_factor():
    ns = NameSpace()
    fortytwo = ns.Number(42)
    factors = ns.factor(fortytwo)
    ref = ns.mul(2, 3, 7)
    assert ref == factors



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import shutil
import sys
from setuptools import setup, Extension


pkg_name = 'symcxx'

SYMCXX_RELEASE_VERSION = os.environ.get('SYMCXX_RELEASE_VERSION', '')  # v*

# Cythonize .pyx file if it exists (not in source distribution)
ext_modules = []


def _read(path, macro='SYMCXX_TYPE', inc_dir='./'):
    for l in open(inc_dir + path, 'rt'):
        if l.startswith('#include'):
            inner_path = l.split('#include')[1]
            inner_path = inner_path.strip().strip('<').strip('>').strip('"')
            for inner_l in _read(inner_path, macro, inc_dir):
                yield inner_l
        if not l.startswith(macro + '('):
            continue
        l = l.split('//')[0]  # strip comments marked with // (misses /* */)
        l = ')'.join(l.split(macro + '(')[1].split(')')[:-1])
        yield tuple([_.strip() for _ in l.split(',')])

if len(sys.argv) > 1 and '--help' not in sys.argv[1:] and sys.argv[1] not in (
        '--help-commands', 'egg_info', 'clean', '--version'):
    try:
        from mako.template import Template
        from mako.exceptions import text_error_template
        from Cython.Build import cythonize
    except ImportError:
        USE_CYTHON = False
    else:
        pyx_path = 'symcxx/_symcxx.pyx'
        template_path = pyx_path + '.mako_template'
        USE_CYTHON = os.path.exists(template_path)

    ext = '.pyx' if USE_CYTHON else '.cpp'
    ext_modules = [Extension(
        'symcxx._symcxx',
        ['symcxx/_symcxx'+ext]
    )]
    if USE_CYTHON:
        stub = 'types_nonatomic_'
        path_stub = 'symcxx/' + stub
        subsd = {}
        subsd['types'] = list(_read('symcxx/types_atomic.inc', inc_dir='./include/'))
        for k in ('unary', 'binary', 'args_stack'):
            subsd[stub+k] = list(_read(path_stub+k+'.inc', inc_dir='./include/'))
            subsd['types'] += subsd[stub+k]

        subsd['_message_for_rendered'] = 'THIS IS A GENERATED FILE DO NOT EDIT'
        try:
            rendered_pyx = Template(open(template_path, 'rt').read()).render(
                **subsd)
        except:
            print(text_error_template().render_unicode())
            raise
        else:
            open(pyx_path, 'wt').write(rendered_pyx)
        ext_modules = cythonize(ext_modules,
                                include_path=['./include'],
                                gdb_debug=True)
    ext_modules[0].sources = [
        'src/basic.cpp', 'src/namespace.cpp'
    ] + ext_modules[0].sources
    ext_modules[0].include_dirs += ['./include']
    ext_modules[0].language = 'c++'
    ext_modules[0].extra_compile_args = ['-std=c++11']


# http://conda.pydata.org/docs/build.html#environment-variables-set-during-the-build-process
if os.environ.get('CONDA_BUILD', '0') == '1':
    try:
        SYMCXX_RELEASE_VERSION = 'v' + open(
            '__conda_version__.txt', 'rt').readline().rstrip()
    except IOError:
        pass

release_py_path = os.path.join(pkg_name, '_release.py')

if len(SYMCXX_RELEASE_VERSION) > 0:
    if SYMCXX_RELEASE_VERSION[0] == 'v':
        TAGGED_RELEASE = True
        __version__ = SYMCXX_RELEASE_VERSION[1:]
    else:
        raise ValueError("Ill formated version")
else:
    TAGGED_RELEASE = False
    # read __version__ attribute from _release.py:
    exec(open(release_py_path).read())


classifiers = [
    "Development Status :: 3 - Alpha",
    'License :: OSI Approved :: BSD License',
    'Operating System :: OS Independent',
    'Topic :: Scientific/Engineering',
    'Topic :: Scientific/Engineering :: Mathematics',
]

tests = [
    'symcxx.tests',
]

long_description = open('README.rst').read()
with open(os.path.join(pkg_name, '__init__.py')) as f:
    short_description = f.read().split('"""')[1]

setup_kwargs = dict(
    name=pkg_name,
    version=__version__,
    description=short_description,
    long_description=long_description,
    classifiers=classifiers,
    author='Björn Dahlgren',
    author_email='bjodah@DELETEMEgmail.com',
    url='https://github.com/bjodah/' + pkg_name,
    license='BSD',
    packages=[pkg_name] + tests,
    setup_requires=['mako'],
    install_requires=['numpy'],
    ext_modules=ext_modules,
)

if __name__ == '__main__':
    try:
        if TAGGED_RELEASE:
            # Same commit should generate different sdist
            # depending on tagged version (set $SYMCXX_RELEASE_VERSION)
            # e.g.:  $ SYMCXX_RELEASE_VERSION=v1.2.3 python setup.py sdist
            # this will ensure source distributions contain the correct version
            shutil.move(release_py_path, release_py_path+'__temp__')
            open(release_py_path, 'wt').write(
                "__version__ = '{}'\n".format(__version__))
        setup(**setup_kwargs)
    finally:
        if TAGGED_RELEASE:
            shutil.move(release_py_path+'__temp__', release_py_path)



# -*- coding: utf-8 -*-
"""
symcxx is a minimal symbolic manipulation python package written in C++.
"""
from __future__ import (absolute_import, division, print_function)

from ._symcxx import PyNameSpace as NameSpace
from ._release import __version__
